@@ -27,6 +27,7 @@
 void xapi_open(void *d);
 void jackt_stop();
 void jackt_start();
+void jackt_toggle();
 void jackt_rewind();
 
 int midi_connected(void);
@@ -40,11 +41,12 @@ extern char *smpte_offset;
 extern int force_redraw; // tell the main event loop that some cfg has changed
 extern int want_letterbox;
 extern int seekflags;
+extern int interaction_override; // disable some options.
 
 extern int movie_width;
 extern int movie_height;
 extern double delay;
-extern double filefps;
+extern double framerate;
 extern int start_fullscreen;
 extern int start_ontop;
 
@@ -120,6 +122,7 @@ static void flip_page(void);
 static OSStatus KeyEventHandler(EventHandlerCallRef nextHandler, EventRef event, void *userData);
 static OSStatus MouseEventHandler(EventHandlerCallRef nextHandler, EventRef event, void *userData);
 static OSStatus WindowEventHandler(EventHandlerCallRef nextHandler, EventRef event, void *userData);
+static OSStatus DialogEventHandler(EventHandlerCallRef nextHandler, EventRef event, void *userData);
 void mac_put_key(UInt32 key, UInt32 charcode);
 OSStatus mac_menu_cmd(OSStatus result, HICommand *acmd);
 
@@ -177,7 +180,8 @@ enum // menubar
         mOSDOffS, 
         mOSDOffF, 
         mSyncJack,
-        mSyncMidi,
+        mSyncJackMidi,
+        mSyncPortMidi,
         mSyncNone,
         mJackPlay,
         mJackStop,
@@ -238,8 +242,9 @@ static void checkMyMenu(void) {
   CheckMenuItem (zoomMenu, 6, want_letterbox);
 
   CheckMenuItem (syncMenu, 1, jack_connected());
-  CheckMenuItem (syncMenu, 4, midi_connected());
-  CheckMenuItem (syncMenu, 5, (!jack_connected() && !midi_connected()));
+  CheckMenuItem (syncMenu, 4, midi_connected() && !strcmp(midi_driver_name(), "PORTMIDI"));
+  CheckMenuItem (syncMenu, 5, midi_connected() && !strcmp(midi_driver_name(), "JACK-MIDI"));
+  CheckMenuItem (syncMenu, 6, (!jack_connected() && !midi_connected()));
 }
 
 // main window setup and painting..
@@ -396,7 +401,8 @@ static void mac_CreateWindow(uint32_t d_width, uint32_t d_height, WindowAttribut
   AppendMenuItemTextWithCFString(syncMenu, CFSTR("JACK"), 0, mSyncJack, &index);
   AppendMenuItemTextWithCFString(syncMenu, CFSTR("Jack Transport"), 0, 0, &index);
   AppendMenuItemTextWithCFString(syncMenu, NULL, kMenuItemAttrSeparator, 0, &index);
-  AppendMenuItemTextWithCFString(syncMenu, CFSTR("Midi"), 0, mSyncMidi, &index);
+  AppendMenuItemTextWithCFString(syncMenu, CFSTR("MTC (portmidi)"), 0, mSyncPortMidi, &index);
+  AppendMenuItemTextWithCFString(syncMenu, CFSTR("MTC (jackmidi)"), 0, mSyncJackMidi, &index);
   AppendMenuItemTextWithCFString(syncMenu, CFSTR("none"), 0, mSyncNone, &index);
 
   //Create Jack Menu
@@ -435,7 +441,21 @@ static void mac_CreateWindow(uint32_t d_width, uint32_t d_height, WindowAttribut
 }
 ////////////////////////////////////////////////////////
 
+DialogRef midiAlertDialog = 0;
+static OSStatus midiAlert(void) {
+  const EventTypeSpec win_events[] = {
+    { kEventClassWindow, kEventWindowClosed },
+    { kEventClassCommand, kEventCommandProcess }
+  };
+  CreateStandardAlert(kAlertNoteAlert, CFSTR("Midi connect failed!"), CFSTR("could not connect to MIDI Port. check the settings."), NULL, &midiAlertDialog);
+  //RunStandardAlert(midiAlertDialog, NULL, NULL);
+  ShowWindow (GetDialogWindow(midiAlertDialog));
+  InstallWindowEventHandler (GetDialogWindow(midiAlertDialog), NewEventHandlerUPP (DialogEventHandler), GetEventTypeCount(win_events), win_events, GetDialogWindow(midiAlertDialog), NULL);
+  RunAppModalLoopForWindow (GetDialogWindow(midiAlertDialog));
+}
+
 // TODO 
+DialogRef settingsDialog = 0;
 static OSStatus NavOpenSettings(void) {
   OSStatus status = 0;
 #if 0
@@ -446,9 +466,14 @@ static OSStatus NavOpenSettings(void) {
   windowAttrs = kWindowNoShadowAttribute | kWindowOpaqueForEventsAttribute;
   CreateNewWindow(kDocumentWindowClass, kWindowStandardDocumentAttributes, &setRect, &theSettings);
 #else
-  DialogRef settingsDialog;
+  const EventTypeSpec win_events[] = {
+    { kEventClassWindow, kEventWindowClosed },
+    { kEventClassCommand, kEventCommandProcess }
+  };
   CreateStandardAlert(kAlertNoteAlert, CFSTR("settings dialog"), CFSTR("not yet implemented. use .xjadeorc file in your $HOME to make settings persist."),NULL, &settingsDialog);
-  RunStandardAlert(settingsDialog, NULL, NULL);
+  InstallWindowEventHandler (GetDialogWindow(settingsDialog), NewEventHandlerUPP (DialogEventHandler), GetEventTypeCount(win_events), win_events, GetDialogWindow(settingsDialog), NULL);
+  ShowWindow (GetDialogWindow(settingsDialog));
+  RunAppModalLoopForWindow (GetDialogWindow(settingsDialog));
 #endif
   return(status);
 }
@@ -478,6 +503,22 @@ CantGetCount:
   return(status);
 }
 
+int ask_close() {
+  DialogRef noticeDialog;
+  DialogItemIndex itemIndex;
+  AlertStdCFStringAlertParamRec inAlertParam;
+  GetStandardAlertDefaultParams(&inAlertParam, kStdCFStringAlertVersionOne);
+  inAlertParam.defaultButton = kAlertStdAlertCancelButton;
+  inAlertParam.defaultText = CFSTR("Quit");
+  inAlertParam.cancelText = CFSTR("Don't Quit");
+
+  CreateStandardAlert(kAlertCautionAlert, CFSTR("Really Quit?"), CFSTR("Application terminaton is beeing blocked by remote control.\nPlease use the controlling application to quit Jadeo.\nQuit anyway?"), &inAlertParam, &noticeDialog);
+  RunStandardAlert(noticeDialog, NULL, &itemIndex);
+  if (itemIndex == 1) return 1;
+  return 0;
+}
+
+
 static OSStatus NavOpenDocument(void) {
   printf("NavOpenDocument\n");
   OSStatus status;
@@ -517,8 +558,8 @@ CantGetDefaultOptions:
 
 ////////////////////////////////////////////////////////
 // osx helpers
-void window_resized() {
-  printf("resized\n");
+void window_resized_mac() {
+  //printf("resized\n");
   
   int padding = 0;
   
@@ -647,7 +688,7 @@ void window_fullscreen() {
     SizeWindow(theWindow, oldWinRect.right, oldWinRect.bottom,1);
     MoveWindow(theWindow, oldWinBounds.left, oldWinBounds.top, 1);
   }
-  window_resized();
+  window_resized_mac();
 }
 
 //default keyboard event handler
@@ -799,6 +840,21 @@ static OSStatus MouseEventHandler(EventHandlerCallRef nextHandler, EventRef even
   return result;
 }
 
+static OSStatus DialogEventHandler(EventHandlerCallRef nextHandler, EventRef event, void *userData)
+{
+  if (midiAlertDialog) {
+    QuitAppModalLoopForWindow(GetDialogWindow(midiAlertDialog));
+    DisposeDialog(midiAlertDialog);
+    midiAlertDialog=0;
+  }
+  if (settingsDialog) {
+    QuitAppModalLoopForWindow(GetDialogWindow(settingsDialog));
+    DisposeDialog(settingsDialog);
+    settingsDialog=0;
+  }
+  return noErr;
+}
+
 //default window event handler
 static OSStatus WindowEventHandler(EventHandlerCallRef nextHandler, EventRef event, void *userData)
 {
@@ -809,11 +865,10 @@ static OSStatus WindowEventHandler(EventHandlerCallRef nextHandler, EventRef eve
   UInt32 class = GetEventClass (event);
   UInt32 kind = GetEventKind (event); 
 
-  result = CallNextEventHandler(nextHandler, event);
-  
   if(class == kEventClassCommand) {
     //printf("kEventClassCommand\n");
     HICommand theHICommand;
+    result = CallNextEventHandler(nextHandler, event);
     GetEventParameter(event, kEventParamDirectObject, typeHICommand, NULL, sizeof( HICommand ), NULL, &theHICommand);
     result = mac_menu_cmd(result, &theHICommand);
 
@@ -831,7 +886,14 @@ static OSStatus WindowEventHandler(EventHandlerCallRef nextHandler, EventRef eve
             GetPortBounds(GetWindowPort(window), &rectPort);
     }   
     switch (kind) {
+      case kEventWindowClose:
+        if ((interaction_override&0x2) == 0 || ask_close()) {
+          result = CallNextEventHandler(nextHandler, event);
+        } 
+        break;
       case kEventWindowClosed:
+        result = CallNextEventHandler(nextHandler, event);
+        // TODO: catch window close
         theWindow = NULL;
         loop_flag=0;
         //exit(0); // XXX
@@ -840,12 +902,14 @@ static OSStatus WindowEventHandler(EventHandlerCallRef nextHandler, EventRef eve
       //resize window
       case kEventWindowZoomed:
       case kEventWindowBoundsChanged:
-        window_resized();
+        result = CallNextEventHandler(nextHandler, event);
+        window_resized_mac();
         flip_page();
-        window_resized();
+        window_resized_mac();
         break;
       default:
-        result = eventNotHandledErr;
+        result = CallNextEventHandler(nextHandler, event);
+        //result = eventNotHandledErr;
         break;
     }
   } 
@@ -1050,7 +1114,7 @@ int open_window_mac (void) {
     SetWindowContentColor(theWindow, &black);
 #endif
   } else {
-    printf("re-set window\n");
+    //printf("re-set window\n");
 #if 1
     HideWindow(theWindow);
     ChangeWindowAttributes(theWindow, ~windowAttrs, windowAttrs);
@@ -1240,7 +1304,7 @@ int open_window_mac (void) {
     window_fullscreen();
   }
 #endif
-  window_resized();
+  window_resized_mac();
   return 0;
 }
 
@@ -1255,7 +1319,6 @@ void close_window_mac (void) {
 }
 
 void newsrc_mac (void) {
-  printf("newsrc\n");
   close_window_mac();
   open_window_mac();
 }
@@ -1266,7 +1329,7 @@ void resize_mac (unsigned int x, unsigned int y) {
           vo_fs = (!(vo_fs)); window_fullscreen();
   }
   SizeWindow(theWindow, x, y ,1);
-  window_resized();
+  window_resized_mac();
 }
 
 void getsize_mac (unsigned int *x, unsigned int *y) { 
@@ -1285,8 +1348,8 @@ void position_mac (int x, int y) {
 
 void getpos_mac (int *x, int *y) { 
   static Rect rect; 
-  GetWindowPortBounds(theWindow, &rect);
-  //GetWindowBounds(theWindow, kWindowContentRgn, &rect);
+  //GetWindowPortBounds(theWindow, &rect); // OSX < 10.4/5 ?!
+  GetWindowBounds(theWindow, kWindowContentRgn, &rect); // OSX 10.6
   if(x)*x=rect.left; if(y)*y=rect.top; 
   printf("get window pos %i %i\n", *x, *y);
 }
@@ -1296,6 +1359,10 @@ void fullscreen_mac (int a) {
   window_fullscreen();
 }
 
+int get_fullscreen_mac() {
+  return (vo_fs);
+}
+
 void mousepointer_mac (int a) {
   if (a>1) a=!mouseHide;
   printf("mouse cursor: %s\n",a?"hide":"show");
@@ -1314,6 +1381,9 @@ void ontop_mac (int a) {
   window_ontop();
 }
 
+int get_ontop_mac() {
+  return(winLevel==2?1:0);
+}
 
 void render_mac (uint8_t *mybuffer) {
 //printf("render\n");
@@ -1356,16 +1426,17 @@ void handle_X_events_mac (void) {
 void mac_put_key(UInt32 key, UInt32 charcode) {
   char c;
   switch (key) {
-    case 0x35: loop_flag=0; return;  // ESCAPE
+    case 0x35: if ((interaction_override&0x1) == 0) loop_flag=0; return;  // ESCAPE
     default: c= (char) charcode;
   }
   switch (c) {
-    case 'q': loop_flag=0; return; 
+    case 'q': if ((interaction_override&0x1) == 0) loop_flag=0; return; 
     case 'a': ontop_mac(winLevel==2?0:1); break;
     case 'f': fullscreen_mac(!vo_fs); break;
     case 'l':  
       want_letterbox = !want_letterbox;
       force_redraw=1;
+      window_resized_mac();
       break;
     case 'o': { //'o' // OSD - offset in frames
       if (OSD_mode&OSD_OFFF) {
@@ -1395,18 +1466,44 @@ void mac_put_key(UInt32 key, UInt32 charcode) {
       force_redraw=1;
     } break;
     case '+': {
+      if ((interaction_override&0x10) != 0 ) break;
       ts_offset++;
       force_redraw=1;
       if (smpte_offset) free(smpte_offset);
       smpte_offset= calloc(15,sizeof(char));
-      frame_to_smptestring(smpte_offset,ts_offset,midi_connected());
+      frame_to_smptestring(smpte_offset,ts_offset);
     } break;
     case '-': {
+      if ((interaction_override&0x10) != 0 ) break;
       ts_offset--;
       force_redraw=1;
       if (smpte_offset) free(smpte_offset);
       smpte_offset= calloc(15,sizeof(char));
-      frame_to_smptestring(smpte_offset,ts_offset,midi_connected());
+      frame_to_smptestring(smpte_offset,ts_offset);
+    } break;
+    case '{': {
+      if ((interaction_override&0x10) != 0 ) break;
+      if (framerate > 0) {
+        ts_offset-= framerate *60;
+      } else {
+        ts_offset-= 25*60;
+      }
+      force_redraw=1;
+      if (smpte_offset) free(smpte_offset);
+      smpte_offset= calloc(15,sizeof(char));
+      frame_to_smptestring(smpte_offset,ts_offset);
+    } break;
+    case '}': {
+      if ((interaction_override&0x10) != 0 ) break;
+      if (framerate > 0) {
+        ts_offset+= framerate *60;
+      } else {
+        ts_offset+= 25*60;
+      }
+      force_redraw=1;
+      if (smpte_offset) free(smpte_offset);
+      smpte_offset= calloc(15,sizeof(char));
+      frame_to_smptestring(smpte_offset,ts_offset);
     } break;
     case 'm': mousepointer_mac(2); break;
     case '.': { //'.' // resize 100%
@@ -1449,8 +1546,10 @@ void mac_put_key(UInt32 key, UInt32 charcode) {
       force_redraw=1;
     } break;
 #endif
+    case 0x8: jackt_rewind(); break;
+    case ' ': jackt_toggle(); break;
     default: 
-      printf("yet unhandled keyboard event: '%c' %0x\n",c,c);
+      printf("yet unhandled keyboard event: '%c' 0x%x\n",c,c);
       break;
   }
   checkMyMenu();
@@ -1460,7 +1559,8 @@ OSStatus mac_menu_cmd(OSStatus result, HICommand *acmd) {
   switch ( acmd->commandID ) {
     case kHICommandQuit:
     case mQuit:
-      loop_flag=0;
+      if ((interaction_override&0x4) == 0 || ask_close())
+        loop_flag=0;
       break;
     case kHICommandAbout:
       break;
@@ -1491,6 +1591,7 @@ OSStatus mac_menu_cmd(OSStatus result, HICommand *acmd) {
     case mKeepAspect:
       want_letterbox = !want_letterbox;
       force_redraw=1;
+      window_resized_mac();
       break;
     case mOSDFrame:
       OSD_mode^=OSD_FRAME; 
@@ -1546,17 +1647,21 @@ OSStatus mac_menu_cmd(OSStatus result, HICommand *acmd) {
 	if (midi_connected()) midi_close();
 #endif
       break;
-    case mSyncMidi: {
+    case mSyncJackMidi:
+    case mSyncPortMidi: {
 	if (jack_connected()) close_jack();
-        char *mp = "-1";
 #ifdef HAVE_MIDI
-	if (!midi_connected()) midi_open(mp);
+	if (midi_connected()) midi_close();
+        if (acmd->commandID == mSyncPortMidi) {
+          midi_choose_driver("portmidi");
+        } else {
+          midi_choose_driver("jack");
+        }
+        char *mp = "-1";
+	midi_open(mp);
 #endif
 	if (!midi_connected()) {
-          printf("MIDI CONNECT FAILED!\n");
-          DialogRef alertDialog;
-          CreateStandardAlert(kAlertNoteAlert, CFSTR("Midi connect failed!"), CFSTR("could not connect to MIDI Port. check the settings."),NULL, &alertDialog);
-          RunStandardAlert(alertDialog, NULL, NULL);
+          midiAlert();
         } 
       }
       break;
