@@ -36,6 +36,9 @@
 	int xoffset = 0;
 #endif
 
+void jackt_toggle();
+void jackt_rewind();
+
 #if (HAVE_LIBXV || HAVE_IMLIB || HAVE_IMLIB2)
 
 Display      		*xj_dpy = NULL;
@@ -166,12 +169,20 @@ void xj_set_ontop (int action) {
 //	net_wm_set_property("_NET_WM_STATE_STAYS_ON_TOP", action);
 }
 
+int xj_get_ontop () {
+	return xj_ontop;
+}
+
 void xj_set_fullscreen (int action) {
 	if (action==2) xj_fullscreen^=1;
 	else xj_fullscreen=action;
 	net_wm_set_property("_NET_WM_STATE_FULLSCREEN", action);
 }
 
+int xj_get_fullscreen () {
+	return (xj_fullscreen);
+}
+
 /* also from mplayer's libvo/x11_common.c - thanks GPL !*/
 void xj_hidecursor (void) {
 	Cursor no_ptr;
@@ -202,9 +213,18 @@ void xj_mousepointer (int action) {
 	else xj_showcursor();
 }
 
-void xj_get_window_pos (int *x,  int *y) {
+void xj_get_window_pos (int *rx,  int *ry) {
 	Window	dummy;
-	XTranslateCoordinates( xj_dpy, xj_win, xj_rwin,0,0,x,y,&dummy );
+	XTranslateCoordinates( xj_dpy, xj_win, xj_rwin, 0, 0, rx, ry, &dummy);
+	while (dummy !=None) {
+		int x, y;
+		XTranslateCoordinates( xj_dpy, xj_win, dummy, 0, 0, &x, &y, &dummy);
+		if (dummy!=None) {
+			(*rx)-=x; (*ry)-=y;
+		} else {
+			(*rx)+=x; (*ry)+=y;
+		}
+	}
 }
 
 void xj_get_window_size (unsigned int *my_Width, unsigned int *my_Height) {
@@ -456,6 +476,8 @@ extern int OSD_mode; // change via keystroke
 extern long ts_offset; 
 extern char    *smpte_offset;
 extern int 	force_redraw; // tell the main event loop that some cfg has changed
+extern int 	interaction_override; // disable some options.
+extern double framerate;
 
 inline void xj_render () { 
 	/* this is the only reference to the global buffer..
@@ -531,7 +553,7 @@ void xj_handle_X_events (void) {
 #endif
 				if (event.xclient.data.l[0] == xj_del_atom) {
 				//	fprintf(stdout, "Window destoyed...\n");
-					loop_flag = 0;
+					if ((interaction_override&0x2) == 0) loop_flag=0;
 #if 0
 				} else if (event.xclient.data.l[0] == xj_a_TakeFocus)  {
 		         		fprintf(stdout, "take X focus!\n");
@@ -581,7 +603,8 @@ void xj_handle_X_events (void) {
 				break;
 			case ButtonRelease:
 				if (event.xbutton.button == 1) {
-					xj_resize(movie_width, movie_height);
+					if ((interaction_override&0x8) == 0)
+					  xj_resize(movie_width, movie_height);
 				} else {
 					unsigned int my_Width,my_Height;
 					xj_get_window_size(&my_Width,&my_Height);
@@ -621,10 +644,12 @@ void xj_handle_X_events (void) {
 					XLookupString(&event.xkey, buf, sizeof(buf), &keySym, &stat);
 					key = ((keySym & 0xff00) != 0 ? ((keySym & 0x00ff) + 256) : (keySym));
 				/* HARDCODED KEY BINDINGS */
-					if        (key == 0x11b) // 'Esc'
-						loop_flag=0;
-					else if   (key == 0x71 ) // 'q'
-						loop_flag=0; 
+					if        (key == 0x11b) { // 'Esc'
+						if ((interaction_override&0x1) == 0) loop_flag=0; 
+					}
+					else if   (key == 0x71 ) {// 'q'
+						if ((interaction_override&0x1) == 0) loop_flag=0; 
+					}
 					else if   (key == 0x61 ) //'a' // always-on-top
 						xj_set_ontop(xj_ontop^=1);
 					else if   (key == 0x66 ) //'f' // fullscreen
@@ -722,17 +747,41 @@ void xj_handle_X_events (void) {
 						my_Height+=step;
 						xj_resize(my_Width, my_Height);
 					} else if (key == 0x2b ) { //'+' // A/V offset
+						if ((interaction_override&0x10) != 0 ) break;
 						ts_offset++;
 						force_redraw=1;
 						if (smpte_offset) free(smpte_offset);
 						smpte_offset= calloc(15,sizeof(char));
-						frame_to_smptestring(smpte_offset,ts_offset,midi_connected());
+						frame_to_smptestring(smpte_offset,ts_offset);
 					} else if (key == 0x2d ) { //'-'  A/V offset
+						if ((interaction_override&0x10) != 0 ) break;
 						ts_offset--;
 						force_redraw=1;
 						if (smpte_offset) free(smpte_offset);
 						smpte_offset= calloc(15,sizeof(char));
-						frame_to_smptestring(smpte_offset,ts_offset,midi_connected());
+						frame_to_smptestring(smpte_offset,ts_offset);
+					} else if (key == 0x7d ) { //'}' // A/V offset
+						if ((interaction_override&0x10) != 0 ) break;
+						if (framerate > 0) {
+							ts_offset+= framerate *60;
+						} else {
+							ts_offset+= 25*60;
+						}
+						force_redraw=1;
+						if (smpte_offset) free(smpte_offset);
+						smpte_offset= calloc(15,sizeof(char));
+						frame_to_smptestring(smpte_offset,ts_offset);
+					} else if (key == 0x7b ) { //'{'  A/V offset
+						if ((interaction_override&0x10) != 0 ) break;
+						if (framerate > 0) {
+							ts_offset-= framerate *60;
+						} else {
+							ts_offset-= 25*60;
+						}
+						force_redraw=1;
+						if (smpte_offset) free(smpte_offset);
+						smpte_offset= calloc(15,sizeof(char));
+						frame_to_smptestring(smpte_offset,ts_offset);
 					} else if (key == 0x6d ) { // 'm' // toggle mouse pointer
 						xj_mousepointer(2);
 #ifdef CROPIMG
@@ -745,6 +794,10 @@ void xj_handle_X_events (void) {
 						if (xoffset>movie_width) xoffset=movie_width;
 						force_redraw=1;
 #endif
+					} else if (key == 0x108 ) { // BACKSPACE
+						jackt_rewind();
+					} else if (key == 0x20 ) { // ' ' // SPACE 
+						jackt_toggle();
 					} else if (want_debug) {
 						printf("unassigned key pressed: '%c' 0x%x\n",key,key);
 					}
@@ -1145,11 +1198,11 @@ int open_window_xv (void) {
 		if (xv_have_I420) {
 			xv_pic_format = FOURCC_I420;
 			if (!want_quiet) 
-				fprintf(stderr,"XV: using YUV420P + Xvideo extention (I420)\n");
+				fprintf(stderr,"XV: using YUV420P + Xvideo extension (I420)\n");
 		} else if (xv_have_YV12) { 
 			xv_pic_format = FOURCC_YV12;
 			if (!want_quiet) 
-				fprintf(stderr,"XV: using YUV420P + Xvideo extention (YV12)\n");
+				fprintf(stderr,"XV: using YUV420P + Xvideo extension (YV12)\n");
 		} else {
 			fprintf(stderr,
 				"Xv: %s: could not find a suitable colormodel in ( ", ad_info[i].name);
