@@ -50,7 +50,6 @@
 #include <unistd.h>
 #include <signal.h>
 
-
 //------------------------------------------------
 // Globals
 //------------------------------------------------
@@ -86,7 +85,7 @@ int               render_fmt = PIX_FMT_YUV420P; ///< needs to be set before call
 double	duration = 1;
 double	framerate = 1;
 long	frames = 1;
-double file_frame_offset = 0;
+int64_t file_frame_offset = 0;
 
 /* Option flags and variables */
 char	*current_file = NULL;
@@ -96,6 +95,14 @@ long	userFrame = 0; // seek to this frame if jack and midi are N/A
 long	dispFrame = 0; // global strorage... = (SMPTE+offset) with boundaries to [0..movie_file_frames]
 int 	force_redraw = 0;
 
+int 	interaction_override = 0; // disable some options.
+/* 1 (bit 0) : ignore 'q' and 'ESC' quit key
+ * 2 (bit 1) : igore window-manager close button
+ * 4 (bit 2) : ignore OSX-Menu QUIT
+ * 8 (bit 3) : ignore left-mouse click
+ * 16(bit 4) : disable '+' '-' '{' '}' offset keys
+ */
+
 int want_quiet   =0;	/* --quiet, --silent */
 int want_debug   =0;	/* -D --debug  (hidden option) */
 int want_verbose =0;	/* --verbose */
@@ -106,12 +113,13 @@ int want_nosplash =0;	/* --nosplash */
 int start_ontop =0;	/* --ontop // -a */
 int start_fullscreen =0;/* --fullscreen // -s */
 int want_letterbox =0;  /* --letterbox -b */
-int want_dropframes =0; /* --dropframes -N  BEWARE! */
-int want_autodrop =1;   /* --nodropframes -n (hidden option) */
+int want_dropframes =0; /* --dropframes -N  -- force using drop-frame timecode */
+int want_autodrop =1;   /* --nodropframes -n (hidden option) -- allow using drop-frame timecode */
 int avoid_lash   =0;	/* --nolash */
 int remote_en =0;	/* --remote, -R */
 int osc_port =0;	/* --osc, -O */
 int mq_en =0;		/* --mq, -Q */
+char *ipc_queue = NULL; /* --ipc, -W */
 int remote_mode =0;	/* 0: undirectional ; >0: bidir
 			 * bitwise enable async-messages 
 			 *  so far only: 
@@ -122,11 +130,12 @@ int remote_mode =0;	/* 0: undirectional ; >0: bidir
 int try_codec =0;	/* --try-codec */
 
 #ifdef HAVE_MIDI
-char midiid[32] = "-2";	/* --midi # -1: autodetect -2: jack*/
+char midiid[128] = "-2";/* --midi # -1: autodetect -2: jack*/
 int midi_clkconvert =0;	/* --midifps [0:MTC|1:VIDEO|2:RESAMPLE] */
-int midi_clkadj =0;	/* --midiclk  */
+char *midi_driver = NULL; /* --mididriver */
 #endif
 
+int have_dropframes =0; /* detected from MTC;  TODO: force to zero if jack or user TC */
 int jack_clkconvert =1; /* --jackfps  - NOT YET IMPLEMENTED
                           [0:audio_frames_per_video_frame
                            1:video-file] */
@@ -137,14 +146,15 @@ lash_client_t *lash_client;
 
 #ifdef HAVE_MACOSX
 // mac GUI defaults
-int 	seekflags    = SEEK_CONTINUOUS; 
-double 	delay = -1; // use file's FPS
+int seekflags    = SEEK_CONTINUOUS; 
 #else
-int 	seekflags    = SEEK_ANY; 
-double 	delay = 0.1; // default update rate 10 Hz
+int seekflags    = SEEK_ANY; 
 #endif
+
+int     midi_clkadj =1;	/* --midiclk  */
 double 	filefps = -1.0; // if > 0 override autodetected video file frame rate
 int	videomode = 0; // --vo <int>  - default: autodetect
+double 	delay = -1; // use file's FPS
 
 
 // On screen display
@@ -185,10 +195,11 @@ static struct option const long_options[] =
   {"offset", no_argument, 0, 'o'},
   {"fps", required_argument, 0, 'f'},
   {"filefps", required_argument, 0, 'F'},
-  {"videomode", required_argument, 0, 'm'},
+  {"videomode", required_argument, 0, 'x'},
   {"vo", required_argument, 0, 'x'},
   {"remote", no_argument, 0, 'R'},
   {"mq", no_argument, 0, 'Q'},
+  {"ipc", required_argument, 0, 'W'},
   {"help", no_argument, 0, 'h'},
   {"version", no_argument, 0, 'V'},
   {"try-codec", no_argument, 0, 't'},
@@ -202,7 +213,9 @@ static struct option const long_options[] =
 #ifdef HAVE_MIDI
   {"midi", required_argument, 0, 'm'},
   {"midifps", required_argument, 0, 'M'},
-  {"midiclk", required_argument, 0, 'C'},
+  {"midi-driver", required_argument, 0, 'd'},
+  {"midiclk", no_argument, 0, 'C'},
+  {"no-midiclk", no_argument, 0, 'c'},
 #endif
 #ifdef HAVE_LIBLO
   {"osc", required_argument, 0, 'O'},
@@ -228,7 +241,8 @@ decode_switches (int argc, char **argv)
 			   "h"	/* help */
 			   "S"	/* nosplash */
 			   "R"	/* stdio remote control */
-			   "Q"	/* message queues */
+			   "Q"	/* POSIX rt-message queues */
+			   "W:"	/* IPC message queues */
 			   "k"	/* keyframes */
 			   "K"	/* anyframe */
 			   "o:"	/* offset */
@@ -243,7 +257,9 @@ decode_switches (int argc, char **argv)
 #ifdef HAVE_MIDI
 			   "m:"	/* midi interface */
 			   "M:"	/* midi clk convert */
+			   "d:"	/* midi driver */
 			   "C"	/* --midiclk */
+			   "c"	/* --no-midiclk */
 #endif
 			   "N"	/* --dropframes */
 			   "n"	/* --nodropframes */
@@ -274,7 +290,7 @@ decode_switches (int argc, char **argv)
 	  want_nosplash = 1;
 	  break;
 	case 'I':		/* --ignorefileoffset */
-	  want_ignstart = 1;
+	  want_ignstart++;
 	  break;
 	case 'P':		/* --avverbose */
 	  want_genpts = 1;
@@ -288,6 +304,10 @@ decode_switches (int argc, char **argv)
 	case 'Q':		/* --mq */
 	  mq_en = 1;
 	  break;
+	case 'W':		/* --ipc */
+	  if (ipc_queue) free(ipc_queue);
+	  ipc_queue = strdup(optarg);
+	  break;
 	case 'O':		/* --avverbose */
 	  osc_port=atoi(optarg);
 	  break;
@@ -336,7 +356,7 @@ decode_switches (int argc, char **argv)
 	  break;
 	case 'F':		/* --filefps */
 	  if(atof(optarg)>0)
-	    filefps = atof(optarg);
+	    filefps = atof(optarg); // TODO: use av_parse_video_frame_rate()
 	  break;
 	case 'f':		/* --fps */
 	  if(atof(optarg)>0)
@@ -349,8 +369,12 @@ decode_switches (int argc, char **argv)
 	  break;
 #ifdef HAVE_MIDI
 	case 'm':		/* --midi */
-	  strncpy(midiid,optarg,32);
-	  midiid[31]=0;
+	  strncpy(midiid,optarg,sizeof(midiid));
+	  midiid[(sizeof(midiid)-1)]=0;
+	  break;
+	case 'd':		/* --midi-driver */
+	  if (midi_driver) free(midi_driver);
+          midi_driver = strdup(optarg);
 	  break;
 	case 'M':		/* --midifps */
           midi_clkconvert = atoi(optarg);
@@ -358,6 +382,9 @@ decode_switches (int argc, char **argv)
 	case 'C':		/* --midiclk */
           midi_clkadj = 1;
 	  break;
+	case 'c':		/* --no-midiclk */
+          midi_clkadj = 0;
+	  break;
 #endif
 	case 'V':
 	  printversion();
@@ -399,13 +426,13 @@ jack video monitor\n", program_name);
 "                            requires x11 or xv videomode and EWMH.\n"
 "  -b, --letterbox           retain apect ratio when scaling (Xv only).\n"
 #ifdef HAVE_MIDI
-"  -C, --midiclk             use midi quarter frames for more exact sync.\n"
+"  -c, --no-midiclk          ignore MTC quarter frames.\n"
 #endif
-"  -f <val>, --fps <val>     video display update fps - default 10.0 fps\n"
+"  -f <val>, --fps <val>     display update freq. - default -1 use file's fps\n"
 "  -i <int> --info <int>     render OnScreenDisplay info: 0:off, %i:frame,\n"
 "                            %i:smpte, %i:both. (use remote ctrl for more opts.)\n"
 "  -I, --ignorefileoffset    set the beginning of the file to SMPTE zero.\n"
-"                            override timestamps of splitted avi or vob files.\n"
+"                            eg. override timestamps of split vob files.\n"
 "  -k, --keyframes           seek to keyframes only\n"
 "  -K, --continuous          decode video source continuously. (extra latency\n"
 "                            when seeking to non-key frames.)\n"
@@ -417,15 +444,21 @@ jack video monitor\n", program_name);
 "",	OSD_FRAME,OSD_SMPTE,OSD_FRAME|OSD_SMPTE); // :)
   printf ("" /* take a breath */
 #ifdef HAVE_MIDI
-#ifdef HAVE_PORTMIDI
-"  -m <int>, --midi <int>    use portmidi instead of jack (-1: autodetect)\n"
+"  -d <name>,                specify midi-driver to use. run 'xjadeo -V' to\n"
+"     --midi-driver <name>   list supported driver(s). <name> is not case-\n"
+"                            sensitive and can be shortened to the first unique\n"
+"                            name. eg '-d j' for jack, '-d alsa-r' for alsa-raw\n"
+"  -m <port>,                use MTC instead of jack-transport\n"
+"      --midi <port>         <port> argument is driver-specific:\n"
+"                            * jack-midi: specify midi-port name to connect to\n"
+"                              or \"\" to not auto-connect.\n"
+"                            * alsa-seq: specify id to connect to. (-1: none)\n"
+"                              eg. -m ardour or -m 80 \n"
+"                            * portmidi: numeric-id; -1: autodetect\n"
 "                            value > -1 specifies a (input) midi port to use\n" 	  
-"                            use -v -m -1 to list midi ports.\n" 	  
-#else /* alsa midi */
-"  -m <port>,                use alsamidi instead of jack\n"
-"      --midi <port>         specify alsa seq id to connect to. (-1: none)\n" 	  
-"                            eg. -m ardour or -m 80 \n"
-#endif /* HAVE_PORTMIDI */
+"                            use '-v -m -1' to list midi-ports.\n" 	  
+"                            * alsa-raw: specify device-name \n"
+"                              eg. -m hw:1,0 or -m 1 \n"
 "  -M <int>,                 how to 'convert' MTC SMPTE to framenumber:\n"
 "      --midifps <int>       0: use framerate of MTC clock (default)\n" 
 "                            2: use video file FPS\n" 
@@ -445,7 +478,9 @@ jack video monitor\n", program_name);
 "  -o <int>, --offset <int>  add/subtract <int> video-frames to/from timecode\n"
 "  -P , --genpts             ffmpeg option - ignore timestamps in the file.\n"
 #ifdef HAVE_MQ
-"  -Q, --mq                  set-up message queues for xjremote\n"
+"  -Q, --mq                  set-up RT message queues for xjremote\n"
+#elif defined HAVE_IPCMSG
+"  -W, --ipc                 set-up IPC message queues for xjremote\n"
 #endif
 #ifdef HAVE_LIBLO
 "  -O <port>, --osc <port>   listen for OSC messages on given port.\n"
@@ -469,20 +504,38 @@ static void printversion (void) {
 #else
   printf ("version %s [ ", VERSION);
 #endif
-#ifndef HAVE_MIDI
-  printf("no MIDI ");
-#else /* have Midi */
-# ifdef HAVE_PORTMIDI
-  printf("portmidi ");
-# else /* alsa midi */
-  printf("alsa-midi ");
-# endif 
-#endif /* HAVE_MIDI */
 #ifdef HAVE_LASH
   printf("LASH ");
 #endif 
+#ifdef HAVE_MQ
+  printf("POSIX-MQueue ");
+#elif defined HAVE_IPCMSG
+  printf("IPC-MSG ");
+#endif
   printf("]\n compiled with LIBAVFORMAT_BUILD 0x%x = %i\n", LIBAVFORMAT_BUILD, LIBAVFORMAT_BUILD);
-  printf(" displays: "
+  printf(" MTC-MIDI: ");
+#ifndef HAVE_MIDI
+  printf("disabled.");
+#else /* have Midi */
+# ifdef HAVE_JACKMIDI
+  printf("jack-midi ");
+# endif 
+# ifdef ALSA_SEQ_MIDI
+  printf("alsa-sequencer ");
+# endif 
+# ifdef HAVE_PORTMIDI
+  printf("portmidi ");
+# endif 
+# ifdef ALSA_RAW_MIDI
+  printf("alsa-raw ");
+# endif 
+# if (defined ALSA_RAW_MIDI || defined HAVE_PORTMIDI || defined ALSA_SEQ_MIDI || defined HAVE_JACKMIDI)
+  printf("(first listed is default)");
+# else
+  printf("no midi-driver available.");
+# endif
+#endif /* HAVE_MIDI */
+  printf("\n displays: "
 #if HAVE_LIBXV
 		"Xv "
 #endif 
@@ -531,6 +584,11 @@ void clean_up (int status) {
   if(remote_en) close_remote_ctrl();
 #ifdef HAVE_MQ
   if(mq_en) close_mq_ctrl();
+#elif defined HAVE_IPCMSG
+  if(ipc_queue) {
+    close_ipcmsg_ctrl();
+    free(ipc_queue);
+  }
 #endif
   shutdown_osc();
 
@@ -539,6 +597,7 @@ void clean_up (int status) {
   close_movie();
 
 #ifdef HAVE_MIDI
+  if (midi_driver) free(midi_driver);
   if (midi_connected()) midi_close(); 
   else
 #endif
@@ -574,6 +633,21 @@ main (int argc, char **argv)
 
   xjadeorc(); // read config files - default values before parsing cmd line.
 
+#ifdef __APPLE__
+{
+  if ( argc >= 2 && strncmp (argv[1], "-psn", 4) == 0 ) {
+    char **gArgv = (char **) malloc(sizeof (char *) * argc);
+    int i;
+    gArgv[0] = argv[0];
+    for (i=1;i<argc-1;++i) {
+      gArgv[i] = argv[i+1];
+    }
+    --argc;
+    argv=gArgv;
+  }
+}
+#endif
+
 #ifdef HAVE_LASH
   for (i=0;i<argc;i++) if (!strncmp(argv[i],"--lash-id",9)) lashed=1;
   lash_args_t *lash_args = lash_extract_args(&argc, &argv);
@@ -599,7 +673,7 @@ main (int argc, char **argv)
   if (videomode < 0) vidoutmode(videomode); // dump modes and exit.
 
   if ((i+1)== argc) movie = argv[i];
-  else if ((remote_en || mq_en) && i==argc) movie = "";
+  else if ((remote_en || mq_en || ipc_queue) && i==argc) movie = "";
 #ifndef HAVE_MACOSX
   else usage (EXIT_FAILURE);
 #else
@@ -649,7 +723,7 @@ main (int argc, char **argv)
 
   if (getvidmode() ==0) {
 	fprintf(stderr,"Could not open display.\n");
-  	if(!remote_en) {  // && !mq_en) { /* TODO: allow windowless startup with MQ ?! */
+  	if(!remote_en) {  // && !mq_en && !ipc_queue) { /* TODO: allow windowless startup with MQ ?! */
 		// TODO: cleanup close midi and file ??
 		close_jack();
 		exit(1);
@@ -659,17 +733,23 @@ main (int argc, char **argv)
   init_moviebuffer();
 
 #ifdef HAVE_MIDI
-  if (atoi(midiid) < -1 ) {
-    open_jack();
-  } else {
+  midi_choose_driver(midi_driver);
+  if (atoi(midiid) >= -1 ) {
+    if (!want_quiet) 
+      printf("using MTC as sync-source.\n");
     midi_open(midiid);
-  }
-#else
-  open_jack();
+  } else 
 #endif
+  {
+    if (!want_quiet) 
+      printf("using JACK-transport as sync source.\n");
+    open_jack();
+  }
 
 #ifdef HAVE_MQ
   if(mq_en) open_mq_ctrl();
+#elif defined HAVE_IPCMSG
+  if(ipc_queue) open_ipcmsg_ctrl(ipc_queue);
 #endif
   if(remote_en) open_remote_ctrl();
 
