@@ -33,6 +33,10 @@
 #include <stdlib.h>
 #include <ctype.h>
 
+/* TODO: compile all available modes (portmidi, jack-midi, alsa-seq & alsa-raw
+ * select on run-time...
+ */
+
 #ifdef HAVE_MIDI
 
 /*
@@ -46,7 +50,8 @@ extern int want_debug;
 extern double framerate;
 extern int midi_clkconvert;
 extern int midi_clkadj;
-extern double 	delay;
+extern int have_dropframes; 
+extern double	delay;
 
 typedef struct {
 	int frame;
@@ -196,7 +201,7 @@ int parse_sysex_urtm (int data, int state, int type) {
 	}
 
 	// type==2 && state6,7,8,9
-	if (state==6 && type ==2 ) { last_tc.hour=(data&0x1f); last_tc.type=(data>>5)&3; } // hour + format
+	if (state==6 && type ==2 ) { last_tc.hour=(data&0x1f); /*last_tc.type=(data>>5)&3*/; } // hour 
 	if (state==7 && type ==2 ) { last_tc.min=(data&0x7f); } // min
 	if (state==8 && type ==2 ) { last_tc.sec=(data&0x7f); } // sec
 	if (state==9 && type ==2 ) { last_tc.frame=(data&0x7f); } // frame
@@ -214,40 +219,6 @@ int parse_sysex_urtm (int data, int state, int type) {
 	return (rv);
 }
 
-/************************************************
- * SMPTE converter.
- *
- * this one does not handle drop frames correctly
- * -> smpte.c
- */
-#if 0
-long convert_smpte_to_frame(smpte now) {
-	long frame =0 ;
-	int fps= 25; 
-	switch(now.type) {
-		case 0: fps=24; break;
-		case 1: fps=25; break;
-		case 2: fps=29; break;
-		case 3: fps=30; break;
-	}
-	switch (midi_clkconvert) {
-		case 2: // force video fps
-			frame = now.frame +  (int)
-				floor(framerate * ( now.sec + 60*now.min + 3600*now.hour));
-		break;
-		case 3: // 'convert' FPS.
-			frame = now.frame + 
-				fps * ( now.sec + 60*now.min + 3600*now.hour);
-			frame = (int) rint(frame * framerate / fps);
-		break;
-		default: // use MTC fps info
-			frame = now.frame + 
-				fps * ( now.sec + 60*now.min + 3600*now.hour);
-	}
-	return(frame);
-}
-#else
-
 long convert_smpte_to_frame (smpte now) {
 	return(smpte_to_frame(
 		now.type,
@@ -258,8 +229,6 @@ long convert_smpte_to_frame (smpte now) {
 		now.day));
 }
 
-#endif
-
 
 /************************************************
  * portmidi 
@@ -270,20 +239,19 @@ long convert_smpte_to_frame (smpte now) {
 #include <portmidi.h>
 #include <porttime.h>
 
-
 typedef void PmQueue;
 PmQueue *Pm_QueueCreate(long num_msgs, long bytes_per_msg);
 PmError Pm_QueueDestroy(PmQueue *queue);
 PmError Pm_Enqueue(PmQueue *queue, void *msg);
 PmError Pm_Dequeue(PmQueue *queue, void *msg);
 
-PmStream * midi = NULL;
+PmStream * pm_midi = NULL;
 
 /* if INPUT_BUFFER_SIZE is 0, PortMidi uses a default value */
 #define INPUT_BUFFER_SIZE 0
 
 
-int midi_detectdevices (int print) {
+int pm_midi_detectdevices (int print) {
 	int midiid=-1;
 	int i;
 
@@ -308,7 +276,7 @@ int midi_detectdevices (int print) {
 	return (midiid);
 }
 
-int midi_check (int midiid) {
+int pm_midi_check (int midiid) {
 	if (midiid < 0 || midiid >=Pm_CountDevices()) {
 		fprintf(stderr,"Error: invalid midi device id.\n");
 		return(-1);
@@ -357,12 +325,12 @@ void process_midi(PtTimestamp timestamp, void *userData)
      
 	/* see if there is any midi input to process */
 	do {
-		result = Pm_Poll(midi);
+		result = Pm_Poll(pm_midi);
 		if (result) {
 			int shift,data;
 			shift=data=0;
 
-			if (Pm_Read(midi, &buffer, 1) == pmBufferOverflow) continue;
+			if (Pm_Read(pm_midi, &buffer, 1) == pmBufferOverflow) continue;
 
 			/* parse only MTC relevant messages */
 			if (Pm_MessageStatus(buffer.message) == 0xf1)
@@ -390,15 +358,15 @@ void process_midi(PtTimestamp timestamp, void *userData)
 }
 
 
-void midi_open(char *midiid) {
+void pm_midi_open(char *midiid) {
 	int midi_input;
-	if (midi) return;
+	if (pm_midi) return;
 
 	midi_input = atoi(midiid);
-	if (want_verbose && midi_input < 0) midi_input = midi_detectdevices(1);
-	else if (midi_input <0 ) midi_input = midi_detectdevices(0);
+	if (want_verbose && midi_input < 0) midi_input = pm_midi_detectdevices(1);
+	else if (midi_input <0 ) midi_input = pm_midi_detectdevices(0);
 
-	if (midi_check(midi_input)) return ;
+	if (pm_midi_check(midi_input)) return ;
 
 	// init smpte
 	tc.type=tc.min=tc.frame=tc.sec=tc.hour=0;
@@ -418,22 +386,22 @@ void midi_open(char *midiid) {
 	Pm_Initialize();
 
 	/* open input device */
-	Pm_OpenInput(&midi, midi_input, NULL, INPUT_BUFFER_SIZE, NULL, NULL);
+	Pm_OpenInput(&pm_midi, midi_input, NULL, INPUT_BUFFER_SIZE, NULL, NULL);
 
 	if (!want_quiet) printf("Midi Input opened.\n");
 
-	Pm_SetFilter(midi, PM_FILT_ACTIVE | PM_FILT_CLOCK);
+	Pm_SetFilter(pm_midi, PM_FILT_ACTIVE | PM_FILT_CLOCK);
 	/* flush the buffer after setting filter, just in case anything got through */
-	while (Pm_Poll(midi)) { Pm_Read(midi, buffer, 1); }
+	while (Pm_Poll(pm_midi)) { Pm_Read(pm_midi, buffer, 1); }
 
 	active = TRUE; 
 }
 
-void midi_close(void) {
+void pm_midi_close(void) {
 	smpte cmd;
 
 	if (!want_quiet) printf("closing midi...");
-	if(!midi) return;
+	if(!pm_midi) return;
 
 	cmd.frame=0xaffe; // shutdown CMD 
 	Pm_Enqueue(main_to_midi, &cmd); 
@@ -443,22 +411,23 @@ void midi_close(void) {
 	Pm_QueueDestroy(midi_to_main);
 	Pm_QueueDestroy(main_to_midi);
 
-	Pm_Close(midi);
-	midi=NULL;
+	Pm_Close(pm_midi);
+	pm_midi=NULL;
+  //have_dropframes = 0; // reset MTC state
 }
 
-int midi_connected(void) {
-	if (midi) return (1);
+int pm_midi_connected(void) {
+	if (pm_midi) return (1);
 	return (0);
 }
 
-long midi_poll_frame (void) {
+long pm_midi_poll_frame (void) {
 	int spin;
 	long frame;
 	static long lastframe = -1 ;
 	static int stopcnt = 0;
     	smpte now;
-	if (!midi) return (0);
+	if (!pm_midi) return (0);
 
 	now.frame=0; // CMD request
 	Pm_Enqueue(main_to_midi, &now); // request data
@@ -469,6 +438,7 @@ long midi_poll_frame (void) {
 	frame = convert_smpte_to_frame(now);
 
 	if(midi_clkadj && (full_tc==0xff)) {
+		double dly = delay>0?delay:(1.0/framerate);
 		//add time that has passed sice last full MTC frame..
 		smpte cmd;
 		cmd.frame=0x4711; // reset-full_tc CMD 
@@ -477,7 +447,7 @@ long midi_poll_frame (void) {
 		if (lastframe != frame) {
 			stopcnt=0;
 			lastframe=frame;
-		} else if (stopcnt++ > (int) ceil(4.0*framerate*delay)) {
+		} else if (stopcnt++ > (int) ceil(4.0*framerate/dly)) {
 			// we expect a full midi MTC every (2.0*framerate/delay) polls
 
 			Pm_Enqueue(main_to_midi, &cmd); // request data
@@ -492,22 +462,202 @@ long midi_poll_frame (void) {
 			printf("\r\t\t\t\t\t\t  |+%g/8\r",diff<0?rint(4.0*(1.75-diff)):diff<2.0?0:rint(4.0*(diff-1.75)));
 	}
 	return(frame);
-	//return(convert_smpte_to_frame(now));
+}
+#endif /* HAVE_PORTMIDI */
+
+#ifdef  HAVE_JACKMIDI /* endif HAVE_PORTMIDI */
+
+/************************************************
+ * jack-midi 
+ */
+
+#include <jack/jack.h>
+#include <jack/transport.h>
+#include <jack/midiport.h>
+
+jack_client_t *jack_midi_client = NULL;
+jack_port_t   *jack_midi_port; 
+
+#define JACK_MIDI_QUEUE_SIZE (1024)
+typedef struct my_midi_event {
+  jack_nframes_t time;
+  size_t size;
+  jack_midi_data_t buffer[16];
+} my_midi_event_t;
+
+my_midi_event_t event_queue[JACK_MIDI_QUEUE_SIZE];
+int queued_events_start = 0;
+int queued_events_end = 0;
+int queued_cycle_id = 0;
+
+void dequeue_jmidi_events(jack_nframes_t until) {
+	int ci = queued_cycle_id;
+	int new=0; // always process data from prev. jack cycles.
+	while (queued_events_start != queued_events_end) {
+		if (queued_events_start == ci ) new=1;
+		if (new && event_queue[queued_events_start].time > until) { 
+			break;
+    }
+
+    my_midi_event_t *ev = &event_queue[queued_events_start];
+
+    if (ev->size==2 && ev->buffer[0] == 0xf1) {
+      parse_timecode(ev->buffer[1]);
+    } else if (ev->size >9 && ev->buffer[0] == 0xf0) {
+      int i;
+      int sysex_type = 0;
+      for (i=1; i<ev->size; ++i) {
+        sysex_type = parse_sysex_urtm(ev->buffer[i],i-1,sysex_type);
+      }
+    }
+		queued_events_start = (queued_events_start +1 ) % JACK_MIDI_QUEUE_SIZE;
+	}
 }
 
-#else  /* endif HAVE_PORTMIDI */
+static int jack_midi_process(jack_nframes_t nframes, void *arg) {
+  void *jack_buf = jack_port_get_buffer(jack_midi_port, nframes);
+  int nevents = jack_midi_get_event_count(jack_buf);
+  int n;
+	queued_cycle_id = queued_events_end;
+
+  for (n=0; n<nevents; n++) {
+		jack_midi_event_t ev;
+    jack_midi_event_get(&ev, jack_buf, n);
+
+    if (ev.size <1 || ev.size > 15) {
+      continue;
+    } else {
+			event_queue[queued_events_end].time = ev.time;
+			event_queue[queued_events_end].size = ev.size;
+			memcpy (event_queue[queued_events_end].buffer, ev.buffer, ev.size);
+			queued_events_end = (queued_events_end +1 ) % JACK_MIDI_QUEUE_SIZE;
+		}
+  }
+  return 0;
+}
+
+void jack_midi_shutdown(void *arg)
+{
+	jack_midi_client=NULL;
+	fprintf (stderr, "jack server shutdown\n");
+}
+
+
+void jm_midi_close(void) {
+	if (jack_midi_client) {
+		jack_deactivate (jack_midi_client);
+		jack_client_close (jack_midi_client);
+  }
+  jack_midi_client = NULL;
+}
+
+void jm_midi_open(char *midiid) {
+  if (midi_connected()) {
+		fprintf (stderr, "xjadeo is alredy connected to jack-midi.\n");
+		return;
+  }
+
+	int i = 0;
+  char jackmidiid[16];
+	do {
+		snprintf(jackmidiid,16,"xjadeo-%i",i);
+		jack_midi_client = jack_client_open (jackmidiid, JackUseExactName, NULL);
+	} while (jack_midi_client == NULL && i++<16);
+
+	if (!jack_midi_client) {
+		fprintf(stderr, "could not connect to jack server.\n");
+    return;
+  }
+
+  jack_on_shutdown (jack_midi_client, jack_midi_shutdown, 0);
+  jack_set_process_callback(jack_midi_client, jack_midi_process, NULL);
+  jack_midi_port = jack_port_register(jack_midi_client, "MTC in", JACK_DEFAULT_MIDI_TYPE, JackPortIsInput , 0);
+
+  if (jack_midi_port == NULL) {
+    fprintf(stderr, "can't register jack-midi-port\n");
+    midi_close(); 
+    return;
+  }
+
+	// init smpte
+	tc.type=tc.min=tc.frame=tc.sec=tc.hour=0;
+	last_tc.type=last_tc.min=last_tc.frame=last_tc.sec=last_tc.hour=0;
+
+  if (jack_activate(jack_midi_client)) {
+    fprintf(stderr, "can't activate jack-midi-client\n");
+    midi_close(); 
+  }
+
+	if (midiid && strlen(midiid)>0) {
+		const char **found_ports = jack_get_ports(jack_midi_client, NULL, JACK_DEFAULT_MIDI_TYPE, JackPortIsOutput);
+		if (found_ports) {
+			int j;
+			for (j = 0; found_ports[j]; ++j) {
+				if (!strncasecmp(found_ports[j], midiid, strlen(midiid))) {
+					if (want_verbose) {
+						printf("JACK-connect '%s' -> '%s'\n", found_ports[j], jack_port_name(jack_midi_port));
+					}
+					if (jack_connect(jack_midi_client, found_ports[j], jack_port_name(jack_midi_port))) {
+						if (!want_quiet) fprintf(stderr,"can not auto-connect jack-midi port.\n");
+					}
+				}
+			}
+		}
+	}
+
+}
+
+int jm_midi_connected(void) {
+	if (jack_midi_client) return (1);
+	return (0);
+}
+
+
+long jm_midi_poll_frame (void) {
+	long frame =0 ;
+	static long lastframe = -1 ;
+	static int stopcnt = 0;
+
+	dequeue_jmidi_events(jack_frames_since_cycle_start(jack_midi_client));
+	frame = convert_smpte_to_frame(last_tc);
+
+	if(midi_clkadj && (full_tc==0xff)) {
+		double dly = delay>0?delay:(1.0/framerate);
+		//add time that has passed sice last full MTC frame.
+		double diff; // unit: smpte-frames.
+		// check if transport is stuck...
+		if (lastframe != frame) {
+			stopcnt=0;
+			lastframe=frame;
+		} else if (stopcnt++ > (int) ceil(4.0*framerate/dly)) {
+			// we expect a full midi MTC every (2.0*framerate/delay) polls
+			full_tc=last_tc.tick=0;
+			if (want_verbose) 
+				printf("\r\t\t\t\t\t\t        -?-\r");
+		}
+		diff= last_tc.tick/4.0;
+
+		if (want_verbose) 
+			// subtract 7 quarter frames latency when running..
+			printf("\r\t\t\t\t\t\t  |+%g/8\r",diff<0?rint(4.0*(1.75-diff)):diff<2.0?0:rint(4.0*(diff-1.75)));
+		frame += (long) rint(diff);
+	}
+	return(frame);
+}
+
+#endif  /* HAVE_JACKMIDI */
 
 /************************************************
  * alsamidi 
  */
 
-#if 0 /* old alsa raw midi  */
+#ifdef ALSA_RAW_MIDI /* old alsa raw midi  */
 
 #include <alsa/asoundlib.h>
 
 static snd_rawmidi_t *amidi= NULL;
-int sysex_state = -1;
-int sysex_type = 0; 
+int ar_sysex_state = -1;
+int ar_sysex_type = 0; 
 
 void amidi_open(char *port_name) {
 	int err=0;
@@ -521,18 +671,19 @@ void amidi_open(char *port_name) {
 	// init smpte
 	tc.type=tc.min=tc.frame=tc.sec=tc.hour=0;
 	last_tc.type=last_tc.min=last_tc.frame=last_tc.sec=last_tc.hour=0;
-        sysex_state = -1;
+        ar_sysex_state = -1;
 
 	snd_rawmidi_nonblock(amidi, 1);
 //	snd_rawmidi_read(amidi, NULL, 0); 
 }
 
-void amidi_close(void) {
+void ar_midi_close(void) {
 	if (!want_quiet) printf("closing alsa midi...");
 	if(!amidi) return;
 	snd_rawmidi_close(amidi);
 	amidi=NULL;
 }
+
  // TODO increase buffer size ( avg: 15Hz * 8 msgs )
  // better: standalone thread
 void amidi_event(void) {
@@ -561,37 +712,32 @@ void amidi_event(void) {
 
 		/* if this is a status byte that's not MIDI_EOX, the sysex
 		 * message is incomplete and there is no more sysex data */
-		if (data & 0x80 && data != MIDI_EOX && data != MIDI_SOX) {sysex_state=-1;}
+		if (data & 0x80 && data != MIDI_EOX && data != MIDI_SOX) {ar_sysex_state=-1;}
 
 		// sysex- universal  real time message f0 7f ... f7 
-	    	if (data == 0xf7) { sysex_state=-1;}
-		else if (sysex_state < 0 && data == 0xf0) { sysex_state=0; sysex_type=0; }
-		else if (sysex_state>=0) {
-			sysex_type = parse_sysex_urtm (data,sysex_state,sysex_type);
-			sysex_state++;
+	    	if (data == 0xf7) { ar_sysex_state=-1;}
+		else if (ar_sysex_state < 0 && data == 0xf0) { ar_sysex_state=0; ar_sysex_type=0; }
+		else if (ar_sysex_state>=0) {
+			ar_sysex_type = parse_sysex_urtm (data,ar_sysex_state,ar_sysex_type);
+			ar_sysex_state++;
 		}
 #endif
 	}
 
 }
 
-long amidi_poll_frame (void) {
+long ar_midi_poll_frame (void) {
 	if (!amidi) return (0);
-
 	amidi_event(); // process midi buffers - get most recent timecode
-	// TODO: add time that has passed since receiving now
 	return(convert_smpte_to_frame(last_tc));
 }
 
-inline long midi_poll_frame (void) { return (amidi_poll_frame() ); }
-inline void midi_close(void) {amidi_close();}
-
-void midi_open(char *midiid) {
+void ar_midi_open(char *midiid) {
 	char devicestring[32];
 	if (atoi(midiid)<0) {
 		if (!want_quiet)
-			fprintf(stdout,"AlsaMIDI does not support autodetection. using default hw:2,0,0\n");
-		snprintf(devicestring,31,"hw:2,0,0");
+			fprintf(stdout,"AlsaMIDI does not support autodetection. using default hw:0,0,0\n");
+		snprintf(devicestring,31,"hw:0,0,0");
 	} else if (isdigit(midiid[0])) {
 		snprintf(devicestring,31,"hw:%s",midiid);
 	} else {
@@ -603,17 +749,19 @@ void midi_open(char *midiid) {
 	amidi_open(devicestring); 
 }
 
-int midi_detectdevices (int print) { 
+int ar_midi_detectdevices (int print) { 
 	if (print) printf("use 'amidi -l' to list Midi ports\n");
 	return(0);
 }
 
-int midi_connected(void) {
+int ar_midi_connected(void) {
 	if (amidi) return (1);
 	return (0);
 }
 
-#else /* 1: alsa raw/sequcer */
+#endif /* ALSA RAW   */
+
+#ifdef ALSA_SEQ_MIDI /* alsa sequcer */
 
 /************************************************
  * alsa seq midi interface 
@@ -632,8 +780,8 @@ pthread_attr_t aseq_pth_attr;
 pthread_mutex_t aseq_lock;
 
 snd_seq_t *seq= NULL;
-int sysex_state = -1;
-int sysex_type = 0; 
+int as_sysex_state = -1;
+int as_sysex_type = 0; 
 int aseq_stop=0; // only modify in main thread. 
 
 void aseq_close(void) {
@@ -641,6 +789,7 @@ void aseq_close(void) {
 	if (!want_quiet) printf("closing alsa midi...");
 	snd_seq_close(seq);
 	seq=NULL;
+  //have_dropframes = 0; // reset MTC state
 }
 
 void aseq_open(char *port_name) {
@@ -696,9 +845,9 @@ void process_seq_event(const snd_seq_event_t *ev) {
 	if (ev->type == SND_SEQ_EVENT_QFRAME) parse_timecode(ev->data.control.value);
 	else if (ev->type == SND_SEQ_EVENT_SYSEX) {
 		unsigned int i; 
-		sysex_type = 0;
+		as_sysex_type = 0;
 		for (i = 1; i < ev->data.ext.len; ++i) {
-			sysex_type = parse_sysex_urtm(((unsigned char*)ev->data.ext.ptr)[i],i-1,sysex_type);
+			as_sysex_type = parse_sysex_urtm(((unsigned char*)ev->data.ext.ptr)[i],i-1,as_sysex_type);
 		}
 	}
 }
@@ -723,7 +872,7 @@ void aseq_event(void) {
 
 }
 
-long aseq_poll_frame (void) {
+long as_midi_poll_frame (void) {
 	long frame =0 ;
 	static long lastframe = -1 ;
 	static int stopcnt = 0;
@@ -734,13 +883,14 @@ long aseq_poll_frame (void) {
 	pthread_mutex_unlock(&aseq_lock);
 
 	if(midi_clkadj && (full_tc==0xff)) {
+		double dly = delay>0?delay:(1.0/framerate);
 		//add time that has passed sice last full MTC frame.
 		double diff; // unit: smpte-frames.
 		// check if transport is stuck...
 		if (lastframe != frame) {
 			stopcnt=0;
 			lastframe=frame;
-		} else if (stopcnt++ > (int) ceil(4.0*framerate*delay)) {
+		} else if (stopcnt++ > (int) ceil(4.0*framerate/dly)) {
 			// we expect a full midi MTC every (2.0*framerate/delay) polls
 			pthread_mutex_lock(&aseq_lock);
 			full_tc=last_tc.tick=0;
@@ -758,8 +908,7 @@ long aseq_poll_frame (void) {
 	return(frame);
 }
 
-inline long midi_poll_frame (void) { return (aseq_poll_frame() ); }
-inline void midi_close(void) {
+inline void as_midi_close(void) {
 	if(!seq) return;
 	aseq_stop =1;
 	pthread_join(aseq_thread,NULL);
@@ -800,7 +949,7 @@ void *aseq_run(void *arg) {
  * Copyright (c) 2005 Clemens Ladisch <clemens@ladisch.de>
  * GPL
  */
-void midi_detectdevices (int print) { 
+void as_midi_detectdevices (int print) { 
 	if (print) {
 		snd_seq_client_info_t *cinfo;
 		snd_seq_port_info_t *pinfo;
@@ -833,10 +982,10 @@ void midi_detectdevices (int print) {
 	}
 }
 
-void midi_open(char *midiid) {
+void as_midi_open(char *midiid) {
 	if (atoi(midiid)<0) {
 		aseq_open(NULL); 
-    		if (want_verbose) midi_detectdevices(1);
+    		if (want_verbose) as_midi_detectdevices(1);
 	} else {
 		aseq_open(midiid); 
 	}
@@ -851,18 +1000,96 @@ void midi_open(char *midiid) {
 	}
 }
 
-int midi_connected(void) {
+int as_midi_connected(void) {
 	if (seq) return (1);
 	return (0);
 }
 
-#endif /*  alsa raw/seq midi  */
+#endif /*  alsa seq midi  */
+
+int  null_midi_connected(void) { return 0;}
+void null_midi_open(char *midiid) {;}
+void null_midi_close(void) {;}
+long null_midi_poll_frame (void) { return 0L;}
+
+#define NULLMIDI 0, &null_midi_open, &null_midi_close, &null_midi_connected, &null_midi_poll_frame
+
+typedef struct {
+	const char *name;
+	int supported; // 1: format compiled in -- 0: not supported 
+	void (*midi_open)(char *);
+	void (*midi_close)(void);
+	int (*midi_connected)(void);
+  long (*midi_poll_frame) (void);
+}midiapi;
+
+const midiapi MA[] = {
+	{ "JACK-MIDI", 
+#ifdef  HAVE_JACKMIDI 
+		1, &jm_midi_open, &jm_midi_close, &jm_midi_connected, &jm_midi_poll_frame
+#else
+		NULLMIDI
+#endif
+	},
+	{ "ALSA-Sequencer",
+#ifdef ALSA_SEQ_MIDI /* alsa sequcer */
+		1, &as_midi_open, &as_midi_close, &as_midi_connected, &as_midi_poll_frame
+#else
+		NULLMIDI
+#endif
+	},
+	{ "PORTMIDI", 
+#ifdef HAVE_PORTMIDI
+		1, &pm_midi_open, &pm_midi_close, &pm_midi_connected, &pm_midi_poll_frame
+#else
+		NULLMIDI
+#endif
+	},
+	{ "ALSA-RAW-MIDI",
+#ifdef ALSA_RAW_MIDI
+		1, &ar_midi_open, &ar_midi_close, &ar_midi_connected, &ar_midi_poll_frame
+#else
+		NULLMIDI
+#endif
+	},
+	{NULL, NULLMIDI}  // the end.
+};
+
+int current_midi_driver = 0;
+
+int midi_choose_driver(char *id) {
+	if (midi_connected()) return -1;
+	int i=0;
+	while (MA[i].name) {
+		if ((id && !strncasecmp(MA[i].name, id, strlen(id))) || (!id && MA[i].supported) ) {
+			current_midi_driver = i;
+			break;
+		}
+		++i;
+	}
+	if (!want_quiet && MA[current_midi_driver].supported) {
+		printf("selected MIDI driver: %s\n", MA[current_midi_driver].name);
+	}
+	return MA[current_midi_driver].supported;
+}
+
+const char *midi_driver_name() {
+	return MA[current_midi_driver].name;
+}
+
+int  midi_connected(void) { return (MA[current_midi_driver].midi_connected());}
+void midi_open(char *midiid) {MA[current_midi_driver].midi_open(midiid);}
+void midi_close(void) {MA[current_midi_driver].midi_close();}
+long midi_poll_frame (void) { return (MA[current_midi_driver].midi_poll_frame());}
 
-#endif /* not HAVE_PORTMIDI = alsamidi */
 
 #else /* HAVE_MIDI */
 
 int midi_connected(void) {
 	return (0);
 }
+
+const char *midi_driver_name() {
+	return "none";
+}
 #endif /* HAVE_MIDI */
