@@ -97,6 +97,7 @@ extern int want_verbose;
 extern int want_letterbox;
 extern int remote_en;
 extern int mq_en;
+extern char *ipc_queue;
 extern int remote_mode;
 
 #ifdef HAVE_MIDI
@@ -115,6 +116,7 @@ extern double 		delay;
 extern double 		filefps;
 extern int		videomode;
 extern int 		seekflags;
+extern int    interaction_override;
 
 // On screen display
 extern char OSD_fontfile[1024]; 
@@ -139,6 +141,10 @@ void remote_printf(int val, const char *format, ...);
 // API commands
 //--------------------------------------------
 
+void xapi_printversion(void *d) {
+	remote_printf(220, "version=%s", VERSION);
+}
+
 void xapi_open(void *d) {
 	char *fn= (char*)d;
 	//printf("open file: '%s'\n",fn);
@@ -221,7 +227,7 @@ void xapi_lvideomodes(void *d) {
 
 void xapi_pletterbox(void *d) {
 	if (want_letterbox)
-		remote_printf(201,"videomode=1 # fixed aspect ratio");
+		remote_printf(201,"letterbox=1 # fixed aspect ratio");
 	else
 		remote_printf(201,"letterbox=0 # free scaling");
 }
@@ -283,7 +289,8 @@ void xapi_swinsize(void *d) {
 void xapi_ontop(void *d) {
 	int action=_NET_WM_STATE_TOGGLE;
 	if (!strcmp(d,"on") || atoi(d)==1) action=_NET_WM_STATE_ADD;
-	else if (!strcmp(d,"off")) action=_NET_WM_STATE_REMOVE;
+	else if (!strcmp(d,"toggle")) action=_NET_WM_STATE_TOGGLE;
+	else if (!strcmp(d,"off") || atoi(d)==0) action=_NET_WM_STATE_REMOVE;
 	remote_printf(100,"ok.");
 	Xontop(action);
 }
@@ -291,11 +298,26 @@ void xapi_ontop(void *d) {
 void xapi_fullscreen(void *d) {
 	int action=_NET_WM_STATE_TOGGLE;
 	if (!strcmp(d,"on") || atoi(d)==1) action=_NET_WM_STATE_ADD;
-	else if (!strcmp(d,"off")) action=_NET_WM_STATE_REMOVE;
+	else if (!strcmp(d,"toggle")) action=_NET_WM_STATE_TOGGLE;
+	else if (!strcmp(d,"off") || atoi(d)==0) action=_NET_WM_STATE_REMOVE;
 	remote_printf(100,"ok.");
 	Xfullscreen(action);
 }
 
+void xapi_pontop(void *d) {
+	if (Xgetontop())
+		remote_printf(201,"windowontop=1 # always on top");
+	else
+		remote_printf(201,"windowontop=0 # normal window stack");
+}
+
+void xapi_pfullscreen(void *d) {
+	if (Xgetfullscreen())
+		remote_printf(201,"fullscreen=1 # full-screen");
+	else
+		remote_printf(201,"fullscreen=0 # windowed");
+}
+
 void xapi_mousepointer(void *d) {
 	int action=2;
 	if (!strcmp(d,"on") || atoi(d)==1) action=1;
@@ -304,6 +326,16 @@ void xapi_mousepointer(void *d) {
 	remote_printf(100,"ok.");
 }
 
+void xapi_poverride(void *d) {
+		remote_printf(201,"override=%i", interaction_override);
+}
+
+void xapi_soverride(void *d) {
+  interaction_override=atoi(d);
+	//remote_printf(100,"ok.");
+	xapi_poverride(NULL);
+}
+
 
 void xapi_swinpos(void *d) {
 	int x,y;
@@ -407,7 +439,7 @@ void xapi_pmheight(void *d) {
 
 void xapi_soffset(void *d) {
   	//long int new = atol((char*)d);
-	ts_offset = smptestring_to_frame((char*)d,midi_connected());
+	ts_offset = smptestring_to_frame((char*)d);
 	remote_printf(101,"offset=%li",(long int) ts_offset);
 }
 
@@ -449,13 +481,13 @@ void xapi_pposition(void *d) {
 
 void xapi_psmpte(void *d) {
 	char smptestr[13];
-	frame_to_smptestring(smptestr,dispFrame,midi_connected());
+	frame_to_smptestring(smptestr,dispFrame);
 	remote_printf(228,"smpte=%s",smptestr);
 }
 
 void xapi_seek(void *d) {
   	//long int new = atol((char*)d);
-	userFrame= smptestring_to_frame((char*)d,midi_connected());
+	userFrame= smptestring_to_frame((char*)d);
 	remote_printf(101,"defaultseek=%li",userFrame);
 }
 
@@ -581,9 +613,19 @@ void xapi_osd_avail(void *d) {
 #endif
 }
 
+void xapi_osd_mode(void *d) {
+	int m = atoi((char*)d);
+	if (m&1) OSD_mode|=OSD_FRAME; else OSD_mode&=~OSD_FRAME;
+	if (m&2) OSD_mode|=OSD_SMPTE; else OSD_mode&=~OSD_SMPTE;
+	if (m&4) OSD_mode|=OSD_TEXT;  else OSD_mode&=~OSD_TEXT;
+	if (m&8) OSD_mode|=OSD_BOX;   else OSD_mode&=~OSD_BOX;
+	remote_printf(100,"set osdmode=%i", (OSD_mode&OSD_FRAME?1:0)|(OSD_mode&OSD_SMPTE?2:0)|(OSD_mode&OSD_TEXT?4:0)|(OSD_mode&OSD_BOX?8:0));
+	force_redraw=1;
+}
+
 void xapi_posd(void *d) {
 #ifdef HAVE_FT
-	remote_printf(201,"osdmode=%i", (OSD_mode&OSD_FRAME?1:0)|(OSD_mode&OSD_SMPTE?2:0)|(OSD_mode&OSD_TEXT?4:0));
+	remote_printf(201,"osdmode=%i", (OSD_mode&OSD_FRAME?1:0)|(OSD_mode&OSD_SMPTE?2:0)|(OSD_mode&OSD_TEXT?4:0)|(OSD_mode&OSD_BOX?8:0));
 	remote_printf(220,"osdfont=%s", OSD_fontfile); 
 	remote_printf(220,"osdtext=%s", OSD_text); 
 #else
@@ -637,6 +679,20 @@ void xapi_midi_status(void *d) {
 #endif
 }
 
+void xapi_smididriver(void *d) {
+#ifdef HAVE_MIDI
+	char *mp = NULL;
+	if (d && strlen(d)>0) mp=d; 
+	if (midi_choose_driver(mp)>0) {
+		remote_printf(100,"ok.");
+	} else {
+		remote_printf(440,"chosen MIDI driver is not supported.");
+	}
+#else
+	remote_printf(499,"midi not available.");
+#endif
+}
+
 void xapi_open_midi(void *d) {
 #ifdef HAVE_MIDI
 	char *mp;
@@ -687,11 +743,7 @@ void xapi_detect_midi(void *d) {
 
 void xapi_pmidilibrary (void *d) {
 #ifdef HAVE_MIDI
-    #ifdef HAVE_PORTMIDI
-	remote_printf(220,"midilib=portmidi");
-    #else
-	remote_printf(220,"midilib=alsaseq");
-    #endif
+	remote_printf(220,"mididrv=%s", midi_driver_name());
 #else
 	remote_printf(499,"midi not available.");
 #endif
@@ -783,7 +835,9 @@ Dcommand cmd_midi[] = {
 	{"disconnect", ": unconect from midi device", NULL, xapi_close_midi, 0 },
 	{"reconnect", ": connect to last specified midi port", NULL, xapi_reopen_midi, 0 },
 	{"status", ": display status of midi connection.", NULL, xapi_midi_status, 0 },
-	{"library", ": display the used midi libaray", NULL, xapi_pmidilibrary, 0 },
+	{"driver ", "<name>: select midi driver.", NULL, xapi_smididriver, 0 },
+	{"driver", ": display the used midi driver", NULL, xapi_pmidilibrary, 0 },
+	{"library", ": alias for 'midi driver' (deprecated)", NULL, xapi_pmidilibrary, 0 },
 	{"sync ", "<int>: set MTC smpte conversion. 0:MTC 2:Video 3:resample", NULL, xapi_smidisync, 0 },
 	{"clk ", "[1|0]: use MTC quarter frames.", NULL, xapi_smidiclk, 0 },
 	{NULL, NULL, NULL , NULL, 0}
@@ -809,6 +863,7 @@ Dcommand cmd_osd[] = {
 	{"font " , "<filename>: use this TTF font file", NULL, xapi_osd_font, 0 },
 	{"box" , ": forces a black box around the OSD", NULL, xapi_osd_box, 0 },
 	{"nobox" , ": make OSD backgroung transparent", NULL, xapi_osd_nobox, 0 },
+	{"mode" , ":<int>: restore OSD as returned by 'get osdcfg'", NULL, xapi_osd_mode, 0 },
 	{NULL, NULL, NULL , NULL, 0}
 };
 
@@ -836,8 +891,10 @@ Dcommand cmd_get[] = {
 	{"osdcfg", ": display status on screen display", NULL, xapi_posd, 0 },
 	{"syncsource", ": display currently used sync source", NULL, xapi_psync, 0 },
 	{"letterbox" , ": query video scaling mode", NULL, xapi_pletterbox, 0 },
-	{"fullscreen" , ": is xjadeo displayed on full screen", NULL, xapi_null, 0 },
-	{"ontop" , ": query window on top mode", NULL, xapi_null, 0 },
+	{"fullscreen" , ": is xjadeo displayed on full screen", NULL, xapi_pfullscreen, 0 },
+	{"ontop" , ": query window on top mode", NULL, xapi_pontop, 0 },
+	{"override", ": query disabled window events", NULL, xapi_poverride , 0 },
+	{"version", ": query xjadeo version", NULL, xapi_printversion , 0 },
 	{NULL, NULL, NULL , NULL, 0}
 };
 
@@ -857,7 +914,9 @@ Dcommand cmd_window[] = {
 	{"size " , "<int>|<int>x<int>: alias for resize", NULL, xapi_swinsize, 0 },
 	{"position " , "<int>x<int>: move window to absolute position", NULL, xapi_swinpos, 0 },
 	{"pos " , "<int>x<int>: alias for 'window position'", NULL, xapi_swinpos, 0 },
-	{"fullscreen " , "[on|off|toggle]: en/disable fullscreen (only XV/x11)", NULL, xapi_fullscreen, 0 },
+	{"xy " , "<int>x<int>: alias for 'window position'", NULL, xapi_swinpos, 0 },
+	{"fullscreen " , "[on|off|toggle]: en/disable fullscreen (only XV/x11/OSX)", NULL, xapi_fullscreen, 0 },
+	{"zoom " , "[on|off|toggle]: alias for 'window fullscreen'", NULL, xapi_fullscreen, 0 },
 	{"letterbox " , "[on|off|toggle]: don't break aspect ratio (only XV/x11-imlib2)", NULL, xapi_sletterbox, 0 },
 	{"mouse " , "[on|off|toggle]: en/disable mouse cursor display (only XV/x11)", NULL, xapi_mousepointer, 0 },
 	{"ontop " , "[on|off|toggle]: en/disable 'on top' (only XV/x11)", NULL, xapi_ontop, 0 },
@@ -871,7 +930,8 @@ Dcommand cmd_set[] = {
 	{"timescale ", "<float> <int>: set timescale and offset", NULL, xapi_stimescale , 0 },
 	{"loop ", "<int>: 0: normal, 1:wrap around", NULL, xapi_sloop , 0 },
 	{"seekmode ", "<1-3>: seek continuous, to any or to keyframes only", NULL, xapi_sseekmode, 0 },
-	{"framerate", "<float>: show frame rate of video file", NULL, xapi_sframerate , 0 },
+	{"framerate ", "<float>: show frame rate of video file", NULL, xapi_sframerate , 0 },
+	{"override ", "<int>: disable window events", NULL, xapi_soverride , 0 },
 	{NULL, NULL, NULL , NULL, 0}
 };
 
@@ -949,7 +1009,7 @@ void exec_remote_cmd_recursive (Dcommand *leave, char *cmd) {
 }
 
 #ifdef HAVE_LASH
-# ifndef HAVE_MQ
+# if !(defined HAVE_MQ || defined HAVE_IPCMSG)
 #warning 
 #warning 
 #warning LASH support - but no POSIX message queues!
@@ -994,8 +1054,10 @@ int remote_read_io(void) {
 	return(0);
 }
 
-#if HAVE_MQ
+#ifdef HAVE_MQ
 # define LOGLEN MQLEN
+#elif defined(HAVE_IPCMSG)
+# define LOGLEN BUFSIZ
 #else
 # define LOGLEN 1023
 #endif
@@ -1034,7 +1096,7 @@ int remote_fd_set(fd_set *fd) {
 // POSIX message queeue
 //--------------------------------------------
 
-#if HAVE_MQ
+#ifdef HAVE_MQ
 // prototypes in mqueue.c
 int  mymq_read(char *data);
 void mymq_reply(int rv, char *str);
@@ -1059,12 +1121,14 @@ int remote_read_mq(void) {
 	int rx;
 	char data[MQLEN];
 	char *t;
+	int rv = -1;
 
 	while ((rx=mymq_read(data)) > 0 ) { 
 		if ((t =  strchr(data, '\n'))) *t='\0';
 		exec_remote_cmd_recursive(cmd_root,data);
+		rv=0;
 	}
-	return(0);
+	return(rv);
 }
 
 void open_mq_ctrl (void) {
@@ -1076,6 +1140,81 @@ void close_mq_ctrl (void) {
 	remote_printf(100, "quit.");
 	mymq_close();
 }
+#elif defined HAVE_IPCMSG
+
+#include <unistd.h>
+#include <string.h>
+#include <pthread.h>
+#include <sys/types.h>
+#include <sys/ipc.h>
+#include <sys/msg.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
+
+struct msgbuf1 {
+	long    mtype;
+	char    mtext[BUFSIZ];
+};
+
+int msqtx = 0;
+int msqrx = 0;
+
+int myipc_reply(int id, char *msg){
+	struct msgbuf1 txbuf;
+	txbuf.mtype=1;
+  snprintf(txbuf.mtext, BUFSIZ, "@%i %s\n", id, msg); // XXX
+  if (msgsnd(msqtx, (const void*) &txbuf, strlen(txbuf.mtext), 0) == -1) {
+		fprintf(stderr, "msgsnd failed., Error = %d: %s\n", errno, strerror(errno));
+		return -1;
+	}
+	return 0;
+}
+
+int remote_read_ipc () {
+	int rv;
+	struct msgbuf1 rxbuf;
+
+	rv = msgrcv(msqrx, (void*) &rxbuf, BUFSIZ, 1, IPC_NOWAIT);
+
+	if(rv < 0 ) {
+		if (errno != EAGAIN && errno != ENOMSG)  {
+			fprintf(stderr, "Msgrcv failed., Error = %d: %s\n", errno, strerror(errno));
+			close_ipcmsg_ctrl();
+			free(ipc_queue);
+			ipc_queue = NULL;
+		}
+		return (-1);
+	}
+
+	// TODO WHILE  LOOP..
+	char *t;
+	if ((t = strchr(rxbuf.mtext, '\n'))) *t='\0';
+	exec_remote_cmd_recursive(cmd_root,rxbuf.mtext);
+	return(0);
+}
+
+int open_ipcmsg_ctrl (const char *queuename) {
+
+  key_t key_tx = ftok (queuename, 'a');
+  key_t key_rx = ftok (queuename, 'b');
+
+	msqrx = msgget(key_rx, IPC_CREAT| S_IRUSR | S_IWUSR);
+	msqtx = msgget(key_tx, IPC_CREAT| S_IRUSR | S_IWUSR);
+	if(msqrx == -1 || msqtx == -1)  {
+		printf("\ngetKey failed., Error = %d: %s\n", errno, strerror(errno));
+		return -1;
+	}
+	return 0;
+}
+
+void close_ipcmsg_ctrl () {
+	remote_printf(100, "quit.");
+#if 0
+	msgctl(msqtx, IPC_RMID, NULL);
+	msgctl(msqrx, IPC_RMID, NULL);
+#endif
+}
 #endif
 
 //--------------------------------------------
@@ -1092,9 +1231,12 @@ void remote_printf(int rv, const char *format, ...) {
 	va_end(arglist);
 
 	text[LOGLEN -1] =0; 
-#if HAVE_MQ
+#ifdef HAVE_MQ
 		/* remote_printf_mq(...) */
 	mymq_reply(rv,text);
+#elif HAVE_IPCMSG
+		/* remote_printf_ipc(...) */
+	if (ipc_queue) myipc_reply(rv,text);
 #endif
 
 		/* remote_printf_io(...) */
