@@ -77,7 +77,8 @@ extern int	render_fmt;
 extern double	duration;
 extern double	framerate;
 extern long	frames;
-extern double	file_frame_offset;
+extern int64_t	file_frame_offset;
+extern int have_dropframes; 
 
 /* Option flags and variables */
 extern char    *current_file;
@@ -96,6 +97,7 @@ extern int	want_ignstart;
 extern int	remote_en;
 extern int	remote_mode;
 extern int	mq_en;
+extern char	*ipc_queue;
 extern double	delay;
 extern int	seekflags;
 
@@ -118,6 +120,7 @@ double 		tpf = 1.0; /* pts/dts increments per video-frame - cached value */
 //--------------------------------------------
 //
 void select_sleep (int usec) {
+	int remote_activity = 0;
 	fd_set fd;
 	int max_fd=0;
 	struct timeval tv = { 0, 0 };
@@ -127,11 +130,19 @@ void select_sleep (int usec) {
 	if (remote_en) {
 		max_fd=remote_fd_set(&fd);
 	}
-
-	if (select(max_fd, &fd, NULL, NULL, &tv)) remote_read_io();
-#if HAVE_MQ
-	if (mq_en) remote_read_mq();
+#if defined HAVE_MQ
+	if (mq_en) {
+		if (!remote_read_mq()) remote_activity=1;
+	}
+#elif defined HAVE_IPCMSG
+	if (ipc_queue) {
+		if (!remote_read_ipc()) remote_activity=1;
+	}
 #endif
+	if (remote_activity) {
+	  tv.tv_sec = 0; tv.tv_usec = 1;
+	}
+	if (select(max_fd, &fd, NULL, NULL, &tv)) remote_read_io();
 }
 
 void event_loop(void) {
@@ -148,7 +159,7 @@ void event_loop(void) {
 
 	while(loop_flag) { /* MAIN LOOP */
 
-		if (loop_run==0) {  // CHECK: && !force_redraw 
+		if (loop_run==0) {  // CHECK: && !force_redraw ?!
 			/* video offline - (eg. window minimized)
 			 * do not update frame 
 			 */
@@ -176,10 +187,10 @@ void event_loop(void) {
 #endif
 
 		offFrame = newFrame + ts_offset;
-
+		long curFrame = dispFrame;
 		display_frame((int64_t)(offFrame), force_redraw);
 
-		if ((remote_en||mq_en) && ((remote_mode&1) || ((remote_mode&2)&& offFrame!=dispFrame)) ) {
+		if ((remote_en||mq_en||ipc_queue) && ((remote_mode&1) || ((remote_mode&2)&& curFrame!=dispFrame)) ) {
 		/*call 	xapi_pposition ?? -> rv:200
 		 * dispFrame is the currently displayed frame 
 		 * = SMPTE + offset
@@ -202,7 +213,7 @@ void event_loop(void) {
 			fprintf(stdout, "frame: smpte:%li    \r", newFrame);
 		#else
 			char tempsmpte[15];
-			frame_to_smptestring(tempsmpte,newFrame,midi_connected());
+			frame_to_smptestring(tempsmpte,newFrame);
 			fprintf(stdout, "smpte: %s f:%li\r", tempsmpte,newFrame);
 		#endif
 			fflush(stdout); 
@@ -277,15 +288,16 @@ int open_movie(char* file_name) {
 	pFrameFMT = NULL;
 	movie_width  = 320;
 	movie_height = 180;
-	framerate = duration = frames = 1;
-	file_frame_offset = 0.0;
+	duration = frames = 1;
+	framerate = 10; // prevent slow reaction to remote-ctl (event loop).
+	file_frame_offset = 0;
 	videoStream=-1;
 	// recalc offset with new framerate
-	if (smpte_offset) ts_offset=smptestring_to_frame(smpte_offset,midi_connected());
+	if (smpte_offset) ts_offset=smptestring_to_frame(smpte_offset);
   
 	/* Open video file */
 	if(av_open_input_file(&pFormatCtx, file_name, NULL, 0, NULL)!=0) {
-		if (!remote_en || !mq_en) //TODO prevent msg only when starting up with no file...
+		if (!remote_en && !mq_en && !ipc_queue) //TODO prevent msg only when starting up with no file...
 			fprintf( stderr, "Cannot open video file %s\n", file_name);
 		return (-1);
 	}
@@ -333,8 +345,13 @@ int open_movie(char* file_name) {
 	}
 	else framerate = 1.0/av_q2d(av_stream->time_base);
 #endif
-
-	// TODO: detect drop frame timecode !
+  
+	// detect drop frame timecode
+  if (fabs(framerate - 30000.0/1001.0) < 0.01) {
+    have_dropframes=1;
+	  if(!want_quiet)
+		  fprintf(stdout, "enabled drop-frame-timecode (use -n to override).\n");
+  }
 
 #if defined(__BIG_ENDIAN__) && (__ppc__) && LIBAVFORMAT_BUILD <= 4616
 // this cast is weird, but it works.. the bytes seem to be in 'correct' order, but the two
@@ -355,11 +372,12 @@ int open_movie(char* file_name) {
 #else
 	tpf = 1.0/(av_q2d(pFormatCtx->streams[videoStream]->time_base)*framerate);
 #endif
-	if (!want_ignstart) 
-		file_frame_offset = (double) framerate*(pFormatCtx->start_time/ (double) AV_TIME_BASE);
+	if (!want_ignstart && pFormatCtx->start_time != AV_NOPTS_VALUE) {
+		file_frame_offset = (int64_t) floor(framerate * (double) pFormatCtx->start_time / (double) AV_TIME_BASE);
+	}
 
 	// recalc offset with new framerate
-	if (smpte_offset) ts_offset=smptestring_to_frame(smpte_offset,midi_connected());
+	if (smpte_offset) ts_offset=smptestring_to_frame(smpte_offset);
 
 	if (!want_quiet) {
 		if (filefps >0 ) 
@@ -368,6 +386,7 @@ int open_movie(char* file_name) {
 			fprintf(stdout, "original frame rate: %g\n", framerate);
 		fprintf(stdout, "length in seconds: %g\n", duration);
 		fprintf(stdout, "total frames: %ld\n", frames);
+		fprintf(stdout, "file start offset: %lld video-frames\n",file_frame_offset);
 	}
   
   // Get a pointer to the codec context for the video stream
@@ -445,14 +464,21 @@ void override_fps (double fps) {
 	tpf = 1.0/(av_q2d(pFormatCtx->streams[videoStream]->time_base)*framerate);
 #endif
 	// recalc offset with new framerate
-	if (smpte_offset) ts_offset=smptestring_to_frame(smpte_offset,midi_connected());
+	if (smpte_offset) ts_offset=smptestring_to_frame(smpte_offset);
 }
 
 int64_t my_avprev = 0; // last recent seeked timestamp
 
 void render_empty_frame(int blit) {
 	// clear image (black / or YUV green)
-	if (render_fmt == PIX_FMT_YUV420P) {
+	if (render_fmt == PIX_FMT_UYVY422) {
+		int i;
+		for (i=0;i<movie_width*movie_height*2;i+=2) {
+		 buffer[i]=0x80;
+		 buffer[i+1]=0x00;
+		}
+	}
+	else if (render_fmt == PIX_FMT_YUV420P) {
 		size_t Ylen  = movie_width * movie_height;
 		memset(buffer,0,Ylen);
 		memset(buffer+Ylen,0x80,Ylen/2);
@@ -460,6 +486,14 @@ void render_empty_frame(int blit) {
 		memset(buffer,0,avpicture_get_size(render_fmt, movie_width, movie_height));
 #ifdef DRAW_CROSS
 	int x,y;
+	if (render_fmt == PIX_FMT_UYVY422)
+	for (x=0,y=0;x< movie_width-1; x++,y= movie_height*x/movie_width) {
+		 int off=(2*x+2*movie_width*y);
+		 buffer[off]=127; buffer[off+1]=127; 
+
+		 off=(2*x+2*movie_width*(movie_height-y-1));
+		 buffer[off]=127; buffer[off+1]=127;
+	}
 	if (render_fmt == PIX_FMT_YUV420P) 
 	for (x=0,y=0;x< movie_width-1; x++,y= movie_height*x/movie_width) {
 		 int yoff=(x+movie_width*y);
@@ -512,7 +546,11 @@ void reset_video_head(AVPacket *packet) {
 	while (!frameFinished) {
 		av_read_frame(pFormatCtx, packet);
 		if(packet->stream_index==videoStream) 
+#if LIBAVCODEC_VERSION_MAJOR < 52 || ( LIBAVCODEC_VERSION_MAJOR == 52 && LIBAVCODEC_VERSION_MINOR < 21)
 		avcodec_decode_video(pCodecCtx, pFrame, &frameFinished, packet->data, packet->size);
+#else
+		avcodec_decode_video2(pCodecCtx, pFrame, &frameFinished, packet);
+#endif
 		if(packet->data) av_free_packet(packet);
 	}
 	my_avprev=dispFrame=0;
@@ -531,8 +569,9 @@ int my_seek_frame (AVPacket *packet, int64_t timestamp) {
 	if (videoStream < 0) return (0); 
 	v_stream = pFormatCtx->streams[videoStream];
 	
-	if (want_ignstart)  // timestamps in the file start counting at ..->start_time 
-		timestamp+= (long int) ( framerate*(pFormatCtx->start_time/ (double) AV_TIME_BASE));
+	if (want_ignstart==1) {
+		timestamp+= file_frame_offset;
+	}
 
 	// TODO: assert  0 < timestamp + ts_offset - (..->start_time)   < length   
 	
@@ -541,13 +580,13 @@ int my_seek_frame (AVPacket *packet, int64_t timestamp) {
 	printf("\nDEBUG: want frame=%li  ", (long int) timestamp);
 # endif
 
-# if 1  // TODO -> -F <ratio>
-	timestamp*=tpf;
-# else	// THIS is eqivalent - or even better at rounding but
-	// does not work with -F <double> 
-	timestamp=av_rescale_q(timestamp,c1_Q,v_stream->time_base); 
-	timestamp=av_rescale_q(timestamp,c1_Q,v_stream->r_frame_rate); //< timestamp/=framerate; 
-# endif
+  if (filefps > 0) {
+		timestamp*=tpf;
+  } else {
+	// does not work with -F <double>, but it's more accurate when rounding ratios
+		timestamp=av_rescale_q(timestamp,c1_Q,v_stream->time_base); 
+		timestamp=av_rescale_q(timestamp,c1_Q,v_stream->r_frame_rate); //< timestamp/=framerate; 
+	}
 
 # ifdef FFDEBUG
 	printf("ts=%li   ##\n", (long int) timestamp);
@@ -573,7 +612,7 @@ int my_seek_frame (AVPacket *packet, int64_t timestamp) {
 		// timestamp-my_avprev > threshold! - Oh well.
 
 		// seek to keyframe *BEFORE* this frame
-		// printf("SEEK!\n");
+		//printf("SEEK: %d\n",timestamp);
 		rv= av_seek_frame(pFormatCtx, videoStream, timestamp, AVSEEK_FLAG_BACKWARD) ;
 		avcodec_flush_buffers(pCodecCtx);
 	} 
@@ -626,6 +665,11 @@ read_frame:
 		av_free_packet(packet);
 		return (0);
 	}
+#if 0
+  if (mtsb != AV_NOPTS_VALUE && mtsb!=0) my_avprev = mtsb;
+	if (mtsb > timestamp) { printf("WRONG want:%lli got:%lli\n", timestamp, mtsb); my_avprev = mtsb;}
+	if (mtsb == timestamp) printf("Right.\n");
+#endif
 
 	if (mtsb >= timestamp) return (1); // ok!
 
@@ -636,7 +680,11 @@ read_frame:
 	//my_avprev= mtsb;
 
 	int frameFinished; 
+#if LIBAVCODEC_VERSION_MAJOR < 52 || ( LIBAVCODEC_VERSION_MAJOR == 52 && LIBAVCODEC_VERSION_MINOR < 21)
 	avcodec_decode_video(pCodecCtx, pFrame, &frameFinished, packet->data, packet->size);
+#else
+	avcodec_decode_video2(pCodecCtx, pFrame, &frameFinished, packet);
+#endif
 	av_free_packet(packet);
 	if (!frameFinished) {
 		if (want_debug)
@@ -666,7 +714,7 @@ void display_frame(int64_t timestamp, int force_update) {
 
 	dispFrame = timestamp;
 	if (OSD_mode&OSD_FRAME) snprintf(OSD_frame,48,"Frame: %li", dispFrame);
-	if (OSD_mode&OSD_SMPTE) frame_to_smptestring(OSD_smpte,dispFrame,midi_connected());
+	if (OSD_mode&OSD_SMPTE) frame_to_smptestring(OSD_smpte,dispFrame);
 
 	if(fFirstTime) {
 		fFirstTime=0;
@@ -695,13 +743,17 @@ void display_frame(int64_t timestamp, int force_update) {
 		#endif
 			frameFinished=0;	
 			if(packet.stream_index==videoStream) 
+#if LIBAVCODEC_VERSION_MAJOR < 52 || ( LIBAVCODEC_VERSION_MAJOR == 52 && LIBAVCODEC_VERSION_MINOR < 21)
 				avcodec_decode_video(pCodecCtx, pFrame, &frameFinished, packet.data, packet.size);
+#else
+				avcodec_decode_video2(pCodecCtx, pFrame, &frameFinished, &packet);
+#endif
 	  
 			/* Did we get a video frame? */
 			if(frameFinished) {
 				/* Convert the image from its native format to FMT */
 #ifdef HAVE_SWSCALE
-				sws_scale(pSWSCtx, pFrame->data, pFrame->linesize, 0, pCodecCtx->height, pFrameFMT->data, pFrameFMT->linesize);
+				sws_scale(pSWSCtx, (const uint8_t * const*)pFrame->data, pFrame->linesize, 0, pCodecCtx->height, pFrameFMT->data, pFrameFMT->linesize);
 #else
 				img_convert((AVPicture *)pFrameFMT, render_fmt, 
 					(AVPicture*)pFrame, pCodecCtx->pix_fmt, pCodecCtx->width, 
@@ -760,6 +812,8 @@ int close_movie() {
 
 	//Close the video file
 	av_close_input_file(pFormatCtx);
+	duration = frames = 1;
+	framerate = 10; // prevent slow reaction to remote-ctl (event loop).
 	return (0);
 }
 
@@ -775,7 +829,11 @@ void do_try_this_file_and_exit(char *movie) {
 	}
 	init_moviebuffer();
 	if (my_seek_frame(&packet, 1)) {
+#if LIBAVCODEC_VERSION_MAJOR < 52 || ( LIBAVCODEC_VERSION_MAJOR == 52 && LIBAVCODEC_VERSION_MINOR < 21)
 		avcodec_decode_video(pCodecCtx, pFrame, &frameFinished, packet.data, packet.size);
+#else
+		avcodec_decode_video2(pCodecCtx, pFrame, &frameFinished, &packet);
+#endif
 	}
 	close_movie();
 	if (!frameFinished) {
