@@ -102,14 +102,19 @@ void Xontop (int a);
 void Xgetsize (unsigned int *x, unsigned int *y);
 void Xposition (int x, int y);
 void Xgetpos (int *x, int *y);
+int Xgetontop (void);
+int Xgetfullscreen (void);
 
 /* remote.c */
 void close_remote_ctrl (void) ;
 void open_remote_ctrl (void);
 void close_mq_ctrl (void) ;
 void open_mq_ctrl (void);
+void close_ipcmsg_ctrl (void) ;
+int  open_ipcmsg_ctrl (const char *);
 int remote_fd_set(fd_set *fd);
 int remote_read_mq(void);
+int remote_read_ipc(void);
 int remote_read_io(void);
 void remote_printf(int val, const char *format, ...);
 
@@ -130,16 +135,18 @@ void close_jack(void);
 int jack_connected(void);
 
 /* smpte.c prototypes */
-long int smptestring_to_frame (char *str, int autodrop);
-void frame_to_smptestring(char *smptestring, long int frame, int autodrop);
+long int smptestring_to_frame (char *str);
+void frame_to_smptestring(char *smptestring, long int frame);
 long int smpte_to_frame(int type, int f, int s, int m, int h, int overflow);
 
 /* midi.c function prototype */
 int midi_connected(void);
+const char *midi_driver_name();
 #ifdef HAVE_MIDI
 long midi_poll_frame (void);
 void midi_open(char *midiid);
 void midi_close(void);
+int midi_choose_driver(char *);
 #endif
 
 /* xjosc.c */
